@@ -4,7 +4,7 @@
 """
 ===============================================================================
 
-    Copyright (C) 2009-2018 Rudolf Cardinal (rudolf@pobox.com).
+    Original code copyright (C) 2009-2018 Rudolf Cardinal (rudolf@pobox.com).
 
     This file is part of cardinal_pythonlib.
 
@@ -22,6 +22,8 @@
 
 ===============================================================================
 
-Support functions to do with the core language.
+**Support functions to do with the core language.**
+
+Not much here...
 
 """
