@@ -4,7 +4,7 @@
 """
 ===============================================================================
 
-    Copyright (C) 2009-2018 Rudolf Cardinal (rudolf@pobox.com).
+    Original code copyright (C) 2009-2018 Rudolf Cardinal (rudolf@pobox.com).
 
     This file is part of cardinal_pythonlib.
 
@@ -21,6 +21,9 @@
     limitations under the License.
 
 ===============================================================================
+
+**Support functions for Alembic, the migration tool for SQLAlchemy.**
+
 """
 
 import logging
@@ -68,13 +71,14 @@ DEFAULT_ALEMBIC_VERSION_TABLE = "alembic_version"
 # =============================================================================
 # http://stackoverflow.com/questions/24622170/using-alembic-api-from-inside-application-code  # noqa
 
-# *** TEST
 def get_head_revision_from_alembic(
         alembic_config_filename: str,
         alembic_base_dir: str = None,
         version_table: str = DEFAULT_ALEMBIC_VERSION_TABLE) -> str:
     """
-    Ask Alembic what its head revision is.
+    Ask Alembic what its head revision is (i.e. where the Python code would
+    like the database to be at).
+
     Arguments:
         alembic_config_filename: config filename
         alembic_base_dir: directory to start in, so relative paths in the
@@ -92,12 +96,15 @@ def get_head_revision_from_alembic(
         return script.get_current_head()
 
 
-# *** TEST
 def get_current_revision(
         database_url: str,
         version_table: str = DEFAULT_ALEMBIC_VERSION_TABLE) -> str:
     """
     Ask the database what its current revision is.
+
+    Arguments:
+        database_url: SQLAlchemy URL for the database
+        version_table: table name for Alembic versions
     """
     engine = create_engine(database_url)
     conn = engine.connect()
@@ -111,6 +118,17 @@ def get_current_and_head_revision(
         alembic_config_filename: str,
         alembic_base_dir: str = None,
         version_table: str = DEFAULT_ALEMBIC_VERSION_TABLE) -> Tuple[str, str]:
+    """
+    Returns a tuple of ``(current_revision, head_revision)``; see
+    :func:`get_current_revision` and :func:`get_head_revision_from_alembic`.
+
+    Arguments:
+        database_url: SQLAlchemy URL for the database
+        alembic_config_filename: config filename
+        alembic_base_dir: directory to start in, so relative paths in the
+            config file work.
+        version_table: table name for Alembic versions
+    """
     # Where we are
     head_revision = get_head_revision_from_alembic(
         alembic_config_filename=alembic_config_filename,
@@ -143,7 +161,29 @@ def upgrade_database(
     "revision" is the destination revision.
 
     See http://alembic.readthedocs.org/en/latest/api/runtime.html
-    but also, in particular, site-packages/alembic/command.py
+    but also, in particular, ``site-packages/alembic/command.py``
+
+    Arguments:
+        alembic_config_filename:
+            config filename
+
+        alembic_base_dir:
+            directory to start in, so relative paths in the config file work
+
+        starting_revision:
+            revision to start at (typically ``None`` to ask the database)
+
+        destination_revision:
+            revision to aim for (typically ``"head"`` to migrate to the latest
+            structure)
+
+        version_table: table name for Alembic versions
+
+        as_sql:
+            run in "offline" mode: print the migration SQL, rather than
+            modifying the database. See
+            http://alembic.zzzcomputing.com/en/latest/offline.html
+
     """
 
     if alembic_base_dir is None:
@@ -180,12 +220,51 @@ def create_database_migration_numbered_style(
         n_sequence_chars: int = 4) -> None:
     """
     Create a new Alembic migration script.
-    The default n_sequence_chars is like Django and gives files like
+    
+    Alembic compares the **state of the database** to the **state of the
+    metadata**, and generates a migration that brings the former up to the
+    latter. (It does **not** compare the most recent revision to the current
+    metadata, so make sure your database is up to date with the most recent
+    revision before running this!)
+    
+    You **must check** that the autogenerated revisions are sensible.
+    
+    How does it know where to look for the database?
+    
+        1. This function changes into the directory of the Alembic ``.ini``
+           file and calls the external program
+        
+           .. code-block:: bash
+        
+                alembic -c ALEMBIC_INI_FILE revision --autogenerate -m MESSAGE --rev-id REVISION_ID
+                
+        2. The Alembic ``.ini`` file points (via the ``script_location``
+           variable) to a directory containing your ``env.py``. Alembic loads
+           this script.
+           
+        3. That script typically works out the database URL and calls further
+           into the Alembic code.        
+           
+   See http://alembic.zzzcomputing.com/en/latest/autogenerate.html.
+
+    Regarding filenames: the default ``n_sequence_chars`` of 4 is like Django
+    and gives files with names like
+
+    .. code-block:: none
+
         0001_x.py, 0002_y.py, ...
 
     NOTE THAT TO USE A NON-STANDARD ALEMBIC VERSION TABLE, YOU MUST SPECIFY
-    THAT IN YOUR env.py (see e.g. CamCOPS).
-    """
+    THAT IN YOUR ``env.py`` (see e.g. CamCOPS).
+
+    Args:
+        alembic_ini_file: filename of Alembic ``alembic.ini`` file
+        alembic_versions_dir: directory in which you keep your Python scripts,
+            one per Alembic revision
+        message: message to be associated with this revision
+        n_sequence_chars: number of numerical sequence characters to use in the
+            filename/revision (see above).
+    """  # noqa
     _, _, existing_version_filenames = next(os.walk(alembic_versions_dir),
                                             (None, None, []))
     existing_version_filenames = [
@@ -235,8 +314,13 @@ def stamp_allowing_unusual_version_table(
         tag: str = None,
         version_table: str = DEFAULT_ALEMBIC_VERSION_TABLE) -> None:
     """
-    Clone of alembic.command.stamp(), but allowing version_table to change.
-    """
+    Stamps the Alembic version table with the given revision; don't run any
+    migrations.
+
+    This function is a clone of ``alembic.command.stamp()``, but allowing
+    ``version_table`` to change. See
+    http://alembic.zzzcomputing.com/en/latest/api/commands.html#alembic.command.stamp
+    """  # noqa
 
     script = ScriptDirectory.from_config(config)
 
